<?php
/**
 * Class HepsiPayPlugin
 * Copyright (c) 2024
 * @author: Alttantire Yazılım Çözümleri <info@alttantire.com>
 * Web: https://www.alttantire.com
 *
 */

if ( ! defined( "ABSPATH" ) ) {
	exit();
}

use Alttantire\HepsiPaySDK\HepsiPayGateway;
use Alttantire\HepsiPaySDK\HepsiPayUtils;

if ( ! class_exists( "HepsiPayPlugin" ) ) {
	class HepsiPayPlugin extends WC_Payment_Gateway {
		public $id;
		public $verified = false;
		private $installmentOption;
		private $frameUrl;
		private $token;
		private $errorMessage;
		
		public function __construct() {
			$this->id                 = ALTTANTIRE_TEXT_DOMAIN;
			$this->method_title       = __(
				"Hepsipay Sanal POS",
				ALTTANTIRE_TEXT_DOMAIN
			);
			$this->method_description = __(
				"Woocommerce için Sanal POS ödeme modülü ",
				ALTTANTIRE_TEXT_DOMAIN
			);
			$this->title              = $this->get_option( "title" );
			$this->description        = $this->get_option( "description" )
				? $this->get_option( "description" )
				: __( "Credit/Debit Card", ALTTANTIRE_TEXT_DOMAIN );
			
			$this->has_fields = true;
			$this->init_settings();
			$this->initApiSettingsFormFields();
			$this->init();
			$this->initApiGateway();
			
		}
		
		private function init() {
			$this->installmentOption = $this->get_option( "installment" )
				? $this->get_option( "installment" )
				: "no";
			
			add_action( "woocommerce_api_" . $this->id, [
				$this,
				"callbackHandler",
			] );
			
			add_action(
				"woocommerce_update_options_payment_gateways_" . $this->id,
				[ $this, "process_admin_options" ]
			);
			
			add_action( "woocommerce_payment_complete", [
				$this,
				"addExtraFeeAfterPayment",
			] );
		}
		
		private function initApiGateway() {
			$merchant_no = $this->get_option( "merchant_no" );
			$terminal_no = $this->get_option( "terminal_no" );
			$store_key   = $this->get_option( "store_key" );
			$environment = $this->get_option( "environment" );
			
			$this->Gateway = new HepsiPayGateway(
				$environment,
				$merchant_no,
				$terminal_no,
				$store_key
			);
		}
		
		public function process_admin_options(): bool {
			$validation = true;
			
			$this::init_settings();
			
			$environment = filter_input(
				INPUT_POST,
				"woocommerce_" . ALTTANTIRE_TEXT_DOMAIN . "_environment",
				FILTER_SANITIZE_SPECIAL_CHARS
			);
			$merchant_no = filter_input(
				INPUT_POST,
				"woocommerce_" . ALTTANTIRE_TEXT_DOMAIN . "_merchant_no",
				FILTER_SANITIZE_SPECIAL_CHARS
			);
			$terminal_no = filter_input(
				INPUT_POST,
				"woocommerce_" . ALTTANTIRE_TEXT_DOMAIN . "_terminal_no",
				FILTER_SANITIZE_SPECIAL_CHARS
			);
			$store_key   = filter_input(
				INPUT_POST,
				"woocommerce_" . ALTTANTIRE_TEXT_DOMAIN . "_store_key",
				FILTER_SANITIZE_SPECIAL_CHARS
			);
			
			if ( empty( $environment ) ) {
				$validation = false;
				$this->add_error( "API URL bilgisini giriniz!!" );
			}
			if ( empty( $merchant_no ) ) {
				$validation = false;
				
				$this->add_error( "Merchant No bilgisini giriniz" );
			}
			if ( empty( $terminal_no ) ) {
				$validation = false;
				
				$this->add_error( "Terminal No bilgisini giriniz" );
			}
			if ( empty( $store_key ) ) {
				$validation = false;
				
				$this->add_error( "Store Key bilgisini giriniz" );
			}
			
			if (
				! $this->checkApiCredentials(
					$environment,
					$merchant_no,
					$terminal_no,
					$store_key
				)
			) {
				$validation = false;
				
				if($this->errorMessage){
					$this->add_error( $this->errorMessage );
					$this->errorMessage="";
				}
				
			}
			
			if ( ! $validation ) {
				echo "<style>.update-nag, .updated { display: none; }</style>";
				
				$this->display_errors();
				
				return false;
			} else {
				return parent::process_admin_options();
			}
		}
		
		private function checkApiCredentials(
			$environment,
			$merchant_no,
			$terminal_no,
			$store_key
		): bool {
			$checkGateway = new HepsiPayGateway(
				$environment,
				$merchant_no,
				$terminal_no,
				$store_key
			);
			
			$checkGateway->set_amount( 1 * 100 );
			$checkGateway->set_account_key( "905551234567" );
			$checkGateway->set_merchant_callback_url(
				get_site_url() . "/?wc-api=" . $this->id
			);
			
			$checkGateway->set_merchant_order_number( "checkApiCredentials" );
			$checkGateway->set_is_guest_user( true );
			$checkGateway->set_force_3ds( true );
			$checkGateway->set_allowed_installments( [ 1 ] );
			$checkGateway->set_pos_alias( "" );
			$checkGateway->set_frame_type( 1 );
			$checkGateway->set_basket_item( [
				"Product"                => "checkApiCredentials",
				"Price"                  => 1 * 100,
				"SubMerchantMemberId"    => null,
				"SubMerchantMemberPrice" => null,
				"CategoryId"             => 0,
				"ExternalId"             => 0,
			] );
			
			try {
				// Check Availability
				$frame_check_availability_response = $checkGateway->check_availability();
				if ( $frame_check_availability_response->Success === true ) {
					return $checkGateway->frame_init()->Success;
				} else {
					return false;
				}
			} catch( InvalidArgumentException $e ) {
				
				if($e->getCode()===0){
					$this->errorMessage = "Sunucu IP Adresi, Hepsipay'de tanımlı değil. Sunucu IP adresini ve API URL bilgisini kontrol ediniz.";
				}
				
				if($e->getCode()===401){
					$this->errorMessage = "Hepsipay API bilgileri geçersiz. Lütfen bilgilerinizi kontrol ediniz!";
				}
				
				return false;
			}
		}
		
		public function generate_custom_html( $key, $data ): string {
			return isset( $data['description'] ) ? $data['description'] : '';
		}
		
		private function show_system_info() {
			
			$analyze = [ "cURL" => false, "file_get_contents" => false, "used_method" => "" ];
			
			if ( function_exists( 'curl_init' ) ) {
				$analyze["cURL"]        = true;
				$analyze["used_method"] = "cURL";
			}
			
			if ( function_exists( 'file_get_contents' ) && ini_get( 'allow_url_fopen' ) ) {
				$analyze["file_get_contents"] = true;
				$analyze["used_method"]       = "fopen";
			}
			
			$green_text_style = "color: #0f9f23;  float: inline-end; padding-left: 10px;";
			$red_text_style = "color: #7e1313;  float: inline-end; padding-left: 10px;";
			$black_text_style = " float: inline-end; padding-left: 10px;";
			
			
			$info = '<p style="padding: 5px; border-bottom: 1px dashed gray">Sunucu IP: ' .
			        "<span style='".$black_text_style."'>" . getenv('SERVER_ADDR') . "</span>".
			        '</p>';
			
			
			$info .= '<p style="padding: 5px; border-bottom: 1px dashed gray">Hepsipay Plugin: ' .
			         ( defined( 'ALTTANTIRE_PLUGIN_VER' ) ?
				         "<span style='".$green_text_style."'>v" . ALTTANTIRE_PLUGIN_VER . "</span>" :
				         "<span style='".$red_text_style."'>Tespit Edilemedi</span>" ) .
			         '</p>';
			
			$info .= '<p style="padding: 5px; border-bottom: 1px dashed gray">Wordpress: ' .
			         "<span style='".$green_text_style."'>v" .
			         get_bloginfo( 'version' ) .
			         "</span>" .
			         '</p>';
			
			$info .= '<p style="padding: 5px; border-bottom: 1px dashed gray">Woocommerce: ' .
			         ( defined( 'WC_VERSION' ) ?
				         "<span style='".$green_text_style."'>v" . WC_VERSION . "</span>" :
				         "<span style='".$red_text_style."'>Tespit Edilemedi</span>" ) .
			         '</p>';
			
			$info .= '<p style="padding: 5px; border-bottom: 1px dashed gray">PHP: ' .
			         ( version_compare( phpversion(), ALTTANTIRE_MIN_PHP_VER, '>=' ) ?
				         "<span style='".$green_text_style."'>v" . phpversion() . "</span>" :
				         "<span style='".$red_text_style."'>v" . phpversion() . "</span>" ) .
			         '</p>';
			
			$info .= '<p style="padding: 5px; border-bottom: 1px dashed gray">cURL: ' .
			         ( $analyze["cURL"] ? "<span style='".$green_text_style."'>Aktif</span>" :
				         "<span style='".$red_text_style."'>Pasif</span>" ) .
			         '</p>';
			
			$info .= '<p style="padding: 5px; border-bottom: 1px dashed gray;">fopen: ' .
			         ( $analyze["file_get_contents"] ?
				         "<span style='".$green_text_style."'>Aktif</span>" :
				         "<span style='".$red_text_style."'>Pasif</span>" ) .
			         '</p>';
			
			$output = '<table class="form-table"><tbody>';
			$output .= '<tr valign="top">';
			$output .= '<th scope="row" class="sss">';
			$output .= '<label for="woocommerce_woo_alttantire_hepsipay_enabled">Sistem Bilgileri </label>';
			$output .= '</th>';
			$output .= '<td class="forminp">';
			$output .= '<fieldset style="width: fit-content; min-width: 400px">' . $info . ' </fieldset>';
			$output .= '</td>';
			$output .= '</tr>';
			$output .= '</tbody></table>';
			
			return $output;
		}
		
		public function initApiSettingsFormFields() {
			$this->form_fields = [
				
				"enabled"     => [
					"title"   => __( "Aktif/Pasif", ALTTANTIRE_TEXT_DOMAIN ),
					"type"    => "checkbox",
					"label"   => __(
						"Hepsipay Sanal POS ile Ödeme Modülü Aktif",
						ALTTANTIRE_TEXT_DOMAIN
					),
					"default" => "no",
				],
				"title"       => [
					"title"       => __( "Başlık", "woocommerce" ),
					"type"        => "text",
					"description" => __(
						"Kullanıcının ödeme sırasında gördüğü ödeme yöntemi başlığını belirtir.",
						ALTTANTIRE_TEXT_DOMAIN
					),
					"default"     => __( "Kredi Kartı / Banka Kartı ile öde - hepsipay", ALTTANTIRE_TEXT_DOMAIN ),
					"desc_tip"    => true,
				],
				"description" => [
					"title"       => __( "Açıklama", ALTTANTIRE_TEXT_DOMAIN ),
					"type"        => "text",
					"description" => "",
					"default"     => __(
						"Hepsipay ile tüm kredi ve banka kartlarıyla ödeme",
						ALTTANTIRE_TEXT_DOMAIN
					),
				],
				"environment" => [
					"title"       => __( "API URL", ALTTANTIRE_TEXT_DOMAIN ),
					"type"        => "text",
					"description" => "",
					"default"     => "",
				],
				"merchant_no" => [
					"title"       => __( "Merchant No", ALTTANTIRE_TEXT_DOMAIN ),
					"type"        => "text",
					"description" => "",
					"default"     => "",
				],
				"terminal_no" => [
					"title"       => __( "Terminal No", ALTTANTIRE_TEXT_DOMAIN ),
					"type"        => "text",
					"description" => "",
					"default"     => "",
				],
				"store_key"   => [
					"title"       => __( "Store Key", ALTTANTIRE_TEXT_DOMAIN ),
					"type"        => "text",
					"description" => "",
					"default"     => "",
				],
				
				'custom_html' => array(
					'title'       => '',
					'type'        => 'custom',
					'description' => $this->show_system_info(),
				),
			
			];
		}
		
		public function validate_fields(): bool {
			return true;
		}
		
		public function payment_fields() {
			?>

            <div id="<?php echo esc_attr( $this->id ); ?>-cc-form"
                 class="wc-credit-card-form wc-payment-form woo-alttantire-payment-form"
                 style="background:transparent;">

                <div class="hepsipayMessage"></div>
                <div id="hepsipayFrameInit"></div>

            </div>
			
			<?php
		}
		
		/**
		 * @throws Exception
		 */
		public function prepare_payment_data(): array {
			global $woocommerce;
			
			$cart = $woocommerce->cart;
			
			$customer       = $cart->get_customer();
			$customer_phone = (int) preg_replace(
				"/\D/",
				"",
				$customer->get_billing_phone()
			);
			
			$customer_phone = substr( $customer_phone, 0, 2 ) !== "90" ? "90" . $customer_phone : $customer_phone;
			
			try {
				if ( ! $customer_phone ) {
					$this->errorMessage = "Size uygun ödeme seçenekleri için cep telefonu numaranızı giriniz.";
					throw new InvalidArgumentException( $this->errorMessage, 101 );
				}
				
				$this->Gateway->set_amount( $cart->total * 100 );
				$this->Gateway->set_account_key( $customer_phone );
				$this->Gateway->set_merchant_callback_url( get_site_url() . "/?wc-api=" . $this->id );
				$this->Gateway->set_merchant_order_number( $this->Gateway->get_guid() );
				$this->Gateway->set_is_guest_user( true );
				$this->Gateway->set_force_3ds( true );
				$this->Gateway->set_allowed_installments( [ 1, 2, 3, 5, 9, 10, 11, 12 ] );
				
				$this->Gateway->set_pos_alias( "" );
				$this->Gateway->set_frame_type( 1 );

				$this->Gateway->set_basket_item( [
					"Product"                => "Sipariş Toplamı",
					"Price"                  => $cart->total * 100,
					"SubMerchantMemberId"    => null,
					"SubMerchantMemberPrice" => null,
					"CategoryId"             =>0,
					"ExternalId"             => 0,
				] );

				// Loop over $cart items
/*				foreach ( $cart->get_cart() as $cart_item ) {
					$product    = $cart_item["data"];
					$price      = $product->get_price();
					$quantity   = $cart_item["quantity"];
					$title      = $product->get_title();
					$product_id = $cart_item["product_id"];
					$terms      = get_the_terms( $product_id, "product_cat" );
					
					$this->Gateway->set_basket_item( [
						"Product"                => $title,
						"Price"                  => $price * $quantity * 100,
						"SubMerchantMemberId"    => null,
						"SubMerchantMemberPrice" => null,
						"CategoryId"             =>
							is_array( $terms ) && ! empty( $terms )
								? $terms[ count( $terms ) - 1 ]->term_id
								: 0,
						"ExternalId"             => $product_id,
					] );
				}*/
				
				// Check Availability
				$frame_check_availability_response = $this->Gateway->check_availability();
				
				if ( $frame_check_availability_response->Success === true ) {
					// Frame Init
					
					$frame_init_response = $this->Gateway->frame_init();
					
					if ( is_object( $frame_init_response ) && $frame_init_response->Success === true ) {
						$this->token    = $frame_init_response->Token;
						$this->frameUrl = $frame_init_response->FrameUrl;
						
						return [
							"frameUrl" => $this->frameUrl,
							"token"    => $this->token,
							"message"  => null,
						];
					} else {
						$this->errorMessage = $frame_init_response->Message;
						throw new InvalidArgumentException( $this->errorMessage, 103 );
					}
					
				} else {
					$this->errorMessage = $frame_check_availability_response->Message;
					throw new InvalidArgumentException( $this->errorMessage, 102 );
				}
			} catch( Exception $e ) {
				if ( $e->getCode() !== 101 ) {
					wc_add_notice( (string) $e->getMessage(), "error" );
				}
				
				return [
					"frameUrl" => null,
					"token"    => null,
					"message"  => $e->getMessage(),
				];
			}
		}
		
		public function process_payment( $order_id ): array {
			$_SESSION["wc_current_order_id"] = $order_id;
			
			return [
				"result"   => "success",
				"messages" => "<script>HepsipaySdk.initPayment();</script>",
			];
		}
		
		public function callbackHandler() {
			echo HepsiPayUtils::getSpinner();
			exit();
		}
		
		/**
		 * @throws Exception
		 */
		public function checkPaymentStatus( $hp_payment_data ) {
			global $woocommerce;
			
			try {
				
				if (
					! is_object( $hp_payment_data ) ||
					! isset( $hp_payment_data->token )
				) {
					throw new InvalidArgumentException( null );
				}
				
				delete_option( "dd_notice" );
				$order_id = $_SESSION["wc_current_order_id"];
				$order    = wc_get_order( $order_id );
				
				$payment_status = $this->Gateway->payment_status( $hp_payment_data->token );
				
				if ( ! is_object( $payment_status ) ) {
					throw new InvalidArgumentException( null );
				}
				
				if ( in_array( $payment_status->Status, [ 4, 5, 12, 14 ] ) ) {
					$order->update_meta_data(
						"_hp_payment_status_data",
						json_encode( $payment_status )
					);
					$order->update_meta_data(
						"_hp_payment_data",
						json_encode( $hp_payment_data )
					);
					
					$order->payment_complete();
					$woocommerce->cart->empty_cart();
					
					return [
						"redirect_url" => $this->get_return_url( $order ),
						"message"      => "",
					];
				}
				
				$err_message = $payment_status->StatusDescription;
				wc_add_notice( (string) $err_message, "error" );
				
				return [
					"redirect_url" => wc_get_checkout_url(),
					"message"      => $err_message,
				];
				
			} catch( Exception $e ) {
				return null;
			}
			
		}
		
		public function addExtraFeeAfterPayment( $order_id ) {
			$order = wc_get_order( $order_id );
			
			$status_data = $order->get_meta( "_hp_payment_status_data" );
			
			if ( isset( $status_data ) ) {
				
				$hp_payment_status_data   = json_decode( $status_data );
				$installment              = $hp_payment_status_data->InstallmentCount ?? 1;
				$payment_type_description = $hp_payment_status_data->PaymentTypeDescription;
				$extra_fee                = $hp_payment_status_data->InterestAmount ?? 0;
				
				if ( $extra_fee >= 0 ) {
					$item_fee = new WC_Order_Item_Fee();
					$item_fee->set_name(
						( $installment > 1
							? $installment . " Taksit"
							: "Tek Çekim" ) .
						" komisyonu (" .
						$payment_type_description .
						")"
					);
					
					$item_fee->set_amount( $extra_fee );
					$item_fee->set_tax_class( "" );
					$item_fee->set_tax_status( "none" );
					$item_fee->set_total( $extra_fee );
					$order->add_item( $item_fee );
					$order->calculate_totals();
				}
			}
		}
		
		public static function handleBillingData() {
			global $woocommerce;
			$gtw = new HepsiPayPlugin( true );
			
			$cart = $woocommerce->cart;
			
			$customer = $cart->get_customer();
			
			$customer->set_billing_email(
				trim(
					filter_input(
						INPUT_POST,
						"billing_email",
						FILTER_SANITIZE_SPECIAL_CHARS
					)
				)
			);
			$customer->set_billing_phone(
				preg_replace(
					"/\D/",
					"",
					filter_input(
						INPUT_POST,
						"billing_phone",
						FILTER_SANITIZE_SPECIAL_CHARS
					)
				)
			);
			
			$response = $gtw->prepare_payment_data();
			echo json_encode( $response );
			
			wp_die();
		}
		
		/**
		 * @throws Exception
		 */
		public static function checkPaymentIsSuccess() {
			$gtw             = new HepsiPayPlugin( true );
			$hp_payment_data = json_decode(
				filter_input( INPUT_POST, "hp_payment_data", FILTER_DEFAULT )
			);
			
			$response = $gtw->checkPaymentStatus( $hp_payment_data );
			echo json_encode( $response );
			wp_die();
		}
		
	}
}
